@@ -1,5 +1,5 @@
+import typing
 import warnings
-
 import numpy as np
 import xarray as xr
 from dask.array.core import map_blocks
@@ -12,6 +12,7 @@ from .missing_values import fort2py_msg, py2fort_msg
 # Dask Wrappers or Internal Wrappers _<funcname>()
 # These Wrapper are executed within dask processes, and should do anything that
 # can benefit from parallel excution.
+supported_types = typing.Union[xr.DataArray, np.ndarray]
 
 
 def _grid_to_triple(x, y, z, msg_py):
@@ -88,7 +89,12 @@ def _triple_to_grid_2d(x_in, y_in, data, x_out, y_out, msg_py):
 # used for any tasks which would not benefit from parallel execution.
 
 
-def grid_to_triple(data, x_in=None, y_in=None, msg_py=None):
+def grid_to_triple(
+    data: supported_types,
+    x_in: supported_types = None,
+    y_in: supported_types = None,
+    msg_py: supported_types = None,
+) -> supported_types:
     """Converts a two-dimensional grid with one-dimensional coordinate
     variables to an array where each grid value is associated with its
     coordinates.
@@ -96,25 +102,23 @@ def grid_to_triple(data, x_in=None, y_in=None, msg_py=None):
     Parameters
     ----------
 
-    data : :class:`xarray.DataArray` or :class:`numpy.ndarray`:
-        Two-dimensional array of size ny x mx containing the data values.
+    data : :class:`xarray.DataArray`, :class:`numpy.ndarray`:
+        Two-dimensional input array of size ny x mx containing the data values.
         Missing values may be present in `data`, but they are ignored.
 
-    x_in : :class:`xarray.DataArray` or :class:`numpy.ndarray`:
-        Coordinates associated with the right dimension of the variable `data`.
-        If `data` is of type :class:`xarray.DataArray` and `x_in` is unspecified,
-        then it comes as the associated coordinate of `data` (if `x_in` is explicitly
-        given, then it will be used for calculations). If `data` is of type
-        :class:`numpy.ndarray`, then it must be explicitly given as input and it
-        must have the same dimension (call it `mx`) as the right dimension of `data`.
+    x_in : :class:`xarray.DataArray`, :class:`numpy.ndarray`:
+        A one-dimensional array that specifies the the right dimension coordinates of
+        the input (`data`).
 
-    y_in : :class:`xarray.DataArray` or :class:`numpy.ndarray`:
-        Coordinates associated with the left dimension of the variable `data`.
-        If `data` is of type :class:`xarray.DataArray` and `y_in` is unspecified,
-        then it comes as the associated coordinate of `data` (if `y_in` is explicitly
-        given, then it will be used for calculations). If `data` is of type
-        :class:`numpy.ndarray`, then it must be explicitly given as input and it
-        must have the same dimension (call it `ny`) as the left dimension of `data`.
+        Note: It should only be explicitly provided when the input (`fi`) is
+        `numpy.ndarray`; otherwise, it should come from `fi.coords`.
+
+    y_in : :class:`xarray.DataArray`, :class:`numpy.ndarray`:
+        A one-dimensional array that specifies the the left dimension coordinates of
+        the input (`data`).
+
+        Note: It should only be explicitly provided when the input (`fi`) is
+        `numpy.ndarray`; otherwise, it should come from `fi.coords`.
 
     msg_py : :obj:`numpy.number`:
         A numpy scalar value that represent a missing value in `data`.
@@ -124,13 +128,11 @@ def grid_to_triple(data, x_in=None, y_in=None, msg_py=None):
     Returns
     -------
 
-    out : :class:`xarray.DataArray`:
+    out : :class:`xarray.DataArray`, :class:`numpy.ndarray`:
         The maximum size of the returned array will be 3 x ld, where ld <= ny x mx.
         If no missing values are encountered in `data`, then ld = ny x mx. If missing
         values are encountered in `data`, they are not returned and hence ld will be
         equal to ny x mx minus the number of missing values found in `data`.
-        The return array will be double if any of the input arrays are double, and float
-        otherwise.
 
     Examples
     --------
@@ -154,108 +156,104 @@ def grid_to_triple(data, x_in=None, y_in=None, msg_py=None):
             output = geocat.comp.grid_to_triple(data, x_in, y_in)
     """
 
-    # TODO: Will need to be revisited after sanity_check work is finished
-
     # ''' Start of boilerplate
+    is_input_xr = True
+
+    # If the input is numpy.ndarray, convert it to xarray.DataArray
     if not isinstance(data, xr.DataArray):
         if (x_in is None) | (y_in is None):
             raise CoordinateError(
-                "ERROR grid_to_triple: Argument `x_in` and `y_in` must be provided explicitly "
-                "unless `data` is an xarray.DataArray.")
+                "grid_to_triple: Argument `x_in` and `y_in` must be provided "
+                "explicitly unless `data` is an xarray.DataArray.")
+
+        is_input_xr = False
 
         data = xr.DataArray(data)
+        data = data.assign_coords({data.dims[-1]: x_in, data.dims[-2]: y_in})
 
-        data = xr.DataArray(
-            data.data,
-            coords={
-                data.dims[-1]: x_in,
-                data.dims[-2]: y_in,
-            },
-            dims=data.dims,
-        )
+    # x_in and y_in should be coming from xarray input coords or assigned
+    # as coords while xarray being initiated from numpy input above
+    x_in = data.coords[data.dims[-1]]
+    y_in = data.coords[data.dims[-2]]
 
-    if (x_in is None):
-        x_in = data.coords[data.dims[-1]]
-    if (y_in is None):
-        y_in = data.coords[data.dims[-2]]
-
-    # Basic sanity checks
+    # Basic validity checks
     if data.ndim != 2:
         raise DimensionError(
-            "ERROR grid_to_triple: `z` must have two dimensions !\n")
+            "grid_to_triple: `data` must have two dimensions !\n")
 
     if x_in.ndim != 1:
         raise DimensionError(
-            "ERROR grid_to_triple: `x_in` must have one dimension !\n")
+            "grid_to_triple: `x_in` must have one dimension !\n")
     elif x_in.shape[0] != data.shape[1]:
         raise DimensionError(
-            "ERROR grid_to_triple: `x_in` must have the same size (call it `mx`) as the "
-            "right dimension of z. !\n")
+            "grid_to_triple: `x_in` must have the same size (call it `mx`) as the "
+            "right dimension of `data`. !\n")
 
     if y_in.ndim != 1:
         raise DimensionError(
-            "ERROR grid_to_triple: `y_in` must have one dimension !\n")
+            "grid_to_triple: `y_in` must have one dimension !\n")
     elif y_in.shape[0] != data.shape[0]:
         raise DimensionError(
-            "ERROR grid_to_triple: `y_in` must have the same size (call it `ny`) as the left dimension of z. !\n"
-        )
+            "grid_to_triple: `y_in` must have the same size (call it `ny`) as the "
+            "left dimension of `data`. !\n")
     # ''' end of boilerplate
 
+    # Inner Fortran wrapper call
     out = _grid_to_triple(x_in.data, y_in.data, data.data, msg_py)
 
-    out = xr.DataArray(out, attrs=data.attrs)
+    # If input was xarray.DataArray, convert output to xarray.DataArray as well
+    if is_input_xr:
+        out = xr.DataArray(out, attrs=data.attrs)
 
     return out
 
 
-def triple_to_grid(data,
-                   x_in,
-                   y_in,
-                   x_out,
-                   y_out,
-                   method=1,
-                   domain=float(1.0),
-                   distmx=None,
-                   missing_value=np.nan,
-                   meta=False):
+def triple_to_grid(
+    data: supported_types,
+    x_in: supported_types,
+    y_in: supported_types,
+    x_out: supported_types,
+    y_out: supported_types,
+    method: int = 1,
+    domain: float = 1.0,
+    distmx: float = None,
+    missing_value: np.number = None,
+    meta: bool = False,
+) -> supported_types:
     """Places unstructured (randomly-spaced) data onto the nearest locations of
     a rectilinear grid.
 
     Parameters
     ----------
 
-    data : :class:`xarray.DataArray`: or :class:`numpy.ndarray`:
+    data : :class:`xarray.DataArray`:, :class:`numpy.ndarray`:
         A multi-dimensional array, whose rightmost dimension is the same
         length as `x_in` and `y_in`, containing the values associated with
         the "x" and "y" coordinates. Missing values may be present but
         will be ignored.
 
-    x_in : :class:`xarray.DataArray`: or :class:`numpy.ndarray`:
-        One-dimensional arrays of the same length containing the coordinates
-        associated with the data values. For geophysical variables, "x"
-        correspond to longitude.
+    x_in : :class:`xarray.DataArray`:, :class:`numpy.ndarray`:
+        A one-dimensional array that specifies the x-coordinate
+        associated with the input (`data`).
 
-    y_in : :class:`xarray.DataArray`: or :class:`numpy.ndarray`:
-        One-dimensional arrays of the same length containing the coordinates
-        associated with the data values. For geophysical variables, "y"
-        correspond to latitude.
+    y_in : :class:`xarray.DataArray`:, :class:`numpy.ndarray`:
+        A one-dimensional array that specifies the y-coordinate
+        associated with the input (`data`).
 
-    x_out : :class:`xarray.DataArray`: or :class:`numpy.ndarray`:
-        A one-dimensional array of length M containing the "x" coordinates
-        associated with the returned two-grid. For geophysical variables,
-        these are longitudes. The coordinates' values must be
-        monotonically increasing.
+    x_out : :class:`xarray.DataArray`:, :class:`numpy.ndarray`:
+        A one-dimensional array of length M containing the x-coordinates
+        associated with the returned two-dimensional grid. The coordinate
+        values must be monotonically increasing.
 
     y_out : :class:`xarray.DataArray`: or :class:`numpy.ndarray`:
-        A one-dimensional array of length N containing the "y" coordinates
-        associated with the returned grid. For geophysical ~variables,
-        these are latitudes. The coordinates' values must be
-        monotonically increasing.
+        A one-dimensional array of length N containing the y-coordinates
+        associated with the returned two-dimensional grid. The coordinate
+        values must be monotonically increasing.
 
     Optional Parameters
     -------------------
 
-    method :
+    method :obj:`int`:
         An integer value that can be 0 or 1. The default value is 1.
         A value of 1 means to use the great circle distance formula
         for distance calculations.
@@ -265,7 +263,7 @@ def triple_to_grid(data,
         high resolution grid) and the number of observations
         relatively small.
 
-    domain :
+    domain :obj:`float`:
         A float value that should be set to a value >= 0. The
         default value is 1.0. If present, the larger this factor,
         the wider the spatial domain allowed to influence grid boundary
@@ -273,7 +271,7 @@ def triple_to_grid(data,
         then values located outside the grid domain specified by
         `x_out` and `y_out` arguments will not be used.
 
-    distmx :
+    distmx :obj:`float`:
         Setting `distmx` allows the user to specify a search
         radius (km) beyond which observations are not considered
         for nearest neighbor. Only applicable when `method` = 1.
@@ -296,11 +294,10 @@ def triple_to_grid(data,
     Returns
     -------
 
-    grid : :class:`xarray.DataArray`:
-        The returned array will be K x N x M, where K
-        represents the leftmost dimensions of `data`, N represent the size of `y_out`,
-        and M represent the size of `x_out` coordinate vectors. It will be of type
-        double if any of the input is double, and float otherwise.
+    grid : :class:`xarray.DataArray`, :class:`numpy.ndarray`:
+        The returned array will be K x N x M, where K represents the leftmost
+        dimensions of `data`, N represent the size of `y_out`,
+        and M represent the size of `x_out` coordinate vectors.
 
     Description
     -----------
@@ -344,95 +341,82 @@ def triple_to_grid(data,
         output = geocat.comp.triple_to_grid(data, x_out, y_out, x_in, y_in)
     """
 
-    # TODO: May need to be revisited after sanity_check work is finished
-
     if (x_in is None) | (y_in is None):
         raise CoordinateError(
-            "ERROR triple_to_grid: Arguments x_in and y_in must always be explicitly provided"
-        )
+            "triple_to_grid: Arguments `x_in` and `y_in` must always be "
+            "explicitly provided!")
 
     # ''' Start of boilerplate
-    # If a Numpy input is given, convert it to Xarray and chunk it just with its dims
+    is_input_xr = True
+
+    # If the input is numpy.ndarray, convert it to xarray.DataArray
     if not isinstance(data, xr.DataArray):
+
+        is_input_xr = False
+
         data = xr.DataArray(data)
-        data_chunk = dict([
-            (k, v) for (k, v) in zip(list(data.dims), list(data.shape))
-        ])
 
-        data = xr.DataArray(
-            data.data,
-            # coords={
-            #     data.dims[-1]: x_in,
-            #     data.dims[-2]: y_in,
-            # },
-            dims=data.dims,  # Comment
-        ).chunk(data_chunk)
-    else:
-        # If an unchunked Xarray input is given, chunk it just with its dims
-        if (data.chunks is None):
-            data_chunk = dict([
-                (k, v) for (k, v) in zip(list(data.dims), list(data.shape))
-            ])
-            data = data.chunk(data_chunk)
-
-        # Ensure the rightmost dimension of input is not chunked
-        elif list(data.chunks)[-1:] != [x_in.shape]:
-            raise ChunkError(
-                "ERROR triple_to_grid: Data must be unchunked along the rightmost two dimensions"
-            )
-
-    # x_in = data.coords[data.dims[-1]]
-    # y_in = data.coords[data.dims[-2]]
-
-    # Basic sanity checks
+    # Basic validity checks
     if x_in.shape[0] != y_in.shape[0] or x_in.shape[0] != data.shape[data.ndim -
                                                                      1]:
         raise DimensionError(
-            "ERROR triple_to_grid: The length of `x_in` and `y_in` must be the same "
-            "as the rightmost dimension of `data` !")
+            "triple_to_grid: The length of `x_in` and `y_in` must be the same "
+            "as the rightmost dimension of `data`!")
     if x_in.ndim > 1 or y_in.ndim > 1:
         raise DimensionError(
-            "ERROR triple_to_grid: `x_in` and `y_in` arguments must be one-dimensional arrays !\n"
+            "triple_to_grid: `x_in` and `y_in` arguments must be one-dimensional arrays!\n"
         )
     if x_out.ndim > 1 or y_out.ndim > 1:
         raise DimensionError(
-            "ERROR triple_to_grid: `x_out` and `y_out` arguments must be one-dimensional array !\n"
+            "triple_to_grid: `x_out` and `y_out` arguments must be one-dimensional array!\n"
         )
 
     if not isinstance(method, int):
         raise TypeError(
-            'ERROR triple_to_grid: `method` arg must be an integer. Set it to either 1 or 0.'
+            'triple_to_grid: `method` arg must be an integer! Set it to either 1 or 0.'
         )
 
     if (method != 0) and (method != 1):
-        raise TypeError(
-            'ERROR triple_to_grid: `method` arg accepts either 0 or 1.')
+        raise TypeError('triple_to_grid: `method` arg accepts either 0 or 1!')
 
     # `distmx` is only applicable when `method`==1
     if method:
         if np.asarray(distmx).size != 1:
             raise ValueError(
-                "ERROR triple_to_grid: Provide a scalar value for `distmx` !")
+                "triple_to_grid: Provide a scalar value for `distmx`!")
     else:
         if distmx is not None:
             raise ValueError(
-                "ERROR triple_to_grid: `distmx` is only applicable when `method`==1 !"
-            )
+                "triple_to_grid: `distmx` is only applicable when `method`==1!")
 
     if np.asarray(domain).size != 1:
-        raise ValueError(
-            "ERROR triple_to_grid: Provide a scalar value for `domain` !")
+        raise ValueError("triple_to_grid: Provide a scalar value for `domain`!")
 
-    # `data` data structure elements and autochunking
+    # If input data is chunked
+    if data.chunks is not None:
+
+        # Ensure the rightmost dimension of `data` is not chunked
+        if list(data.chunks)[-1:] != [x_in.shape]:
+            raise ChunkError(
+                "triple_to_grid: Data must be unchunked along the rightmost dimension!"
+            )
+
+    # NOTE: Auto-chunking, regardless of what chunk sizes were given by the user, seems
+    # to be explicitly needed in this function because:
+    # The Fortran routine for this function is implemented assuming it would be looped
+    # across the leftmost dimensions of the input (`data`), i.e. on one-dimensional
+    # chunks of size that is equal to the rightmost dimension of `data`, which is the
+    # same as length of `x_in` or `y_in`.
+
+    # Generate chunks of {'dim_0': 1, 'dim_1': 1, ..., 'dim_n': x_in.shape}
     data_chunks = list(data.dims)
     data_chunks[:-1] = [
         (k, 1) for (k, v) in zip(list(data.dims)[:-1],
-                                 list(data.chunks)[:-1])
+                                 list(data.shape)[:-1])
     ]
     data_chunks[-1:] = [
-        (k, v[0])
-        for (k, v) in zip(list(data.dims)[-1:],
-                          list(data.chunks)[-1:])
+        (k, v) for (k, v) in zip(list(data.dims)[-1:],
+                                 list(data.shape)[-1:])
     ]
     data_chunks = dict(data_chunks)
     data = data.chunk(data_chunks)
@@ -447,6 +431,7 @@ def triple_to_grid(data,
     grid_coords[data.dims[-2]] = y_out
     # ''' end of boilerplate
 
+    # Inner Fortran wrapper call
     grid = map_blocks(
         _triple_to_grid,
         data.data,
@@ -470,28 +455,32 @@ def triple_to_grid(data,
     grid = grid.reshape(grid_shape)
 
     if meta:
-        # grid = xr.DataArray(grid.compute(), attrs=data.attrs, dims=data.dims, coords=grid_coords)
+        # grid = xr.DataArray(grid, attrs=data.attrs, dims=data.dims, coords=grid_coords)
         import warnings
         warnings.warn(
             "WARNING triple_to_grid: Retention of metadata is not yet supported; "
             "it will thus be ignored in the output!")
     # else:
-    #     grid = xr.DataArray(grid.compute(), coords=grid_coords)
+    #     grid = xr.DataArray(grid, coords=grid_coords)
 
-    grid = xr.DataArray(grid.compute())
+    # If input was xarray.DataArray, convert output to xarray.DataArray as well
+    if is_input_xr:
+        grid = xr.DataArray(grid)
 
     return grid
 
 
 # TODO: Revisit for implementing this function after deprecating geocat.ncomp
 def triple_to_grid_2d(x_in, y_in, data, x_out, y_out, msg_py):
-    # ''' signature:  grid = triple2grid2d(x,y,data,x_out,y_out,msg_py)
-    pass
+    warnings.warn(
+        "triple_to_grid_2d function name and signature will be deprecated soon "
+        "in a future version. Use `grid_to_triple` instead!",
+        PendingDeprecationWarning)
+
+    return triple_to_grid(data, x_in, y_in, x_out, y_out, missing_value=msg_py)
 
 
 # Transparent wrappers for geocat.ncomp backwards compatibility
-
-
 def grid2triple(x_in, y_in, data, msg_py):
     warnings.warn(
         "grid2triple function name and signature will be deprecated soon "
@@ -501,6 +490,7 @@ def grid2triple(x_in, y_in, data, msg_py):
     return grid_to_triple(data, x_in, y_in, msg_py)
 
 
+# Transparent wrappers for geocat.ncomp backwards compatibility
 def triple2grid(x_in, y_in, data, x_out, y_out, **kwargs):
     warnings.warn(
         "triple2grid function name and signature will be deprecated soon "
